@@ -1,11 +1,12 @@
-//! This module implements a simplified, yet safe version of
+//! This module implements a simplified but safe version of
 //! [`scopeguard`](https://crates.io/crates/scopeguard).
 
+use std::mem::ManuallyDrop;
 use std::ops::{Deref, DerefMut};
 
-/// [`ExitGuard`] captures the environment and invokes the defined closure at the end of the scope.
-pub(super) struct ExitGuard<T, F: FnOnce(T)> {
-    drop_callback: Option<(T, F)>,
+/// [`ExitGuard`] captures the environment and invokes a defined closure at the end of the scope.
+pub(crate) struct ExitGuard<T, F: FnOnce(T)> {
+    drop_callback: ManuallyDrop<(T, F)>,
 }
 
 impl<T, F: FnOnce(T)> ExitGuard<T, F> {
@@ -13,7 +14,7 @@ impl<T, F: FnOnce(T)> ExitGuard<T, F> {
     #[inline]
     pub(crate) const fn new(captured: T, drop_callback: F) -> Self {
         Self {
-            drop_callback: Some((captured, drop_callback)),
+            drop_callback: ManuallyDrop::new((captured, drop_callback)),
         }
     }
 }
@@ -21,9 +22,8 @@ impl<T, F: FnOnce(T)> ExitGuard<T, F> {
 impl<T, F: FnOnce(T)> Drop for ExitGuard<T, F> {
     #[inline]
     fn drop(&mut self) {
-        if let Some((c, f)) = self.drop_callback.take() {
-            f(c);
-        }
+        let (c, f) = unsafe { ManuallyDrop::take(&mut self.drop_callback) };
+        f(c);
     }
 }
 
@@ -32,13 +32,13 @@ impl<T, F: FnOnce(T)> Deref for ExitGuard<T, F> {
 
     #[inline]
     fn deref(&self) -> &Self::Target {
-        unsafe { &self.drop_callback.as_ref().unwrap_unchecked().0 }
+        &self.drop_callback.0
     }
 }
 
 impl<T, F: FnOnce(T)> DerefMut for ExitGuard<T, F> {
     #[inline]
     fn deref_mut(&mut self) -> &mut Self::Target {
-        unsafe { &mut self.drop_callback.as_mut().unwrap_unchecked().0 }
+        &mut self.drop_callback.0
     }
 }
